------------------------------------------------
-- Creating a Table with an Image BLOB column
------------------------------------------------
connect scott/tiger
create table image_blob_table (id            number primary key, 
                               width         integer,
                               height        integer,
                               contentLength integer, 
                               mimeType      varchar2(20),
                               image_blob    BLOB);

------------------------------------------------
-- Importing Images into the Database
------------------------------------------------
connect / as sysdba
--create or replace directory imagedir as '/home/alamb/quickstart/';
-- For Windows:
create or replace directory imagedir as 'c:\quickstart';
grant read on directory imagedir to scott;

connect scott/tiger
-- imports the file named <filename> in IMAGEDIR into image_blob_table in a new 
-- row. The new row has the id identifier set to <dest_id>
create or replace procedure image_blob_import(dest_id number, filename varchar2) is
  img_blob BLOB;
  ctx raw(64) := null;
begin
   delete from image_blob_table where id = dest_id;
   insert into image_blob_table (id, image_blob) values (dest_id, empty_blob())
     returning image_blob into img_blob;

   ORDSYS.ORDImage.importFrom(ctx, img_blob, 'file', 'IMAGEDIR', filename);
   update image_blob_table set image_blob=img_blob where id=dest_id;
end;
/

call image_blob_import(1,'flowers.jpg');
call image_blob_import(2,'goats.gif');


------------------------------------------------
-- Populating height, width, contentLength and
-- mimeType columns in image_blob_table
------------------------------------------------
connect scott/tiger
-- sets the height, width, contentLength and mimeType columns for all rows
-- in the image_blob_table that have null mimetypes using the 
-- ORDImage.getProperties() method
create or replace procedure image_blob_getproperties is
  unused_attributes        CLOB;
  img_mimeType             varchar2(32);
  img_width                integer;
  img_height               integer;
  img_contentLength        integer;
  unused_fileFormat        varchar2(32);
  unused_contentFormat     varchar2(32);
  unused_compressionFormat varchar2(32);
begin
 for rec in (select id, image_blob from image_blob_table where mimeType is null) loop
     ORDSYS.ORDImage.getProperties(rec.image_blob, 
                                   unused_attributes, 
                                   img_mimeType,
                                   img_width, 
                                   img_height, 
                                   unused_fileFormat, 
                                   unused_compressionFormat, 
                                   unused_contentFormat,
                                   img_contentLength);
     update image_blob_table 
       set width=img_width, 
           height=img_height,
           contentLength=img_contentLength,
           mimeType = img_mimeType
       where id=rec.id;
  end loop;
  commit;
end;
/

call image_blob_getProperties();

------------------------------------------------
-- Selecting and Viewing Image Properties
------------------------------------------------
connect scott/tiger
select id, height, width, mimeType, contentLength from image_blob_table;

------------------------------------------------
-- Creating Thumbnails and Changing Formats
------------------------------------------------
connect scott/tiger
-- Generates a new BLOB row with id=<dest_id> based on <verb> and the BLOB
-- in the row with id=<source_id>.
-- NOTE: assumes <source_id> not equal to <dest_id>
create or replace procedure image_blob_processCopy(source_id number, dest_id number, verb varchar2) is
    src_blob BLOB;
    dst_blob BLOB;
begin
  delete from image_blob_table where id = dest_id;
  insert into image_blob_table (id, image_blob) 
    values (dest_id, empty_blob());

  select image_blob into src_blob 
    from image_blob_table 
    where id = source_id;
  select image_blob into dst_blob
    from image_blob_table 
    where id = dest_id for update;

  ORDSYS.ORDImage.processCopy(src_blob, verb, dst_blob);
  update image_blob_table set image_blob = dst_blob where id = dest_id;
end;
/

-- Scale flowers.jpg to 10% into row with id=3
call image_blob_processcopy(1,3,'scale=.1');

-- convert goats.gif to grayscale jpeg thumbnail into row with id=4
call image_blob_processcopy(2,4,'fileformat=jfif contentformat=8bitgray maxscale=100 100');

-- update the metadata for the newly created image rows
call image_blob_getProperties();

-- admire our handiwork
select id, height, width, mimeType, contentLength from image_blob_table;

-- Convert to PNG if Extproc is not set up correctly.
call image_blob_processcopy(2,5,'fileformat=pngf contentformat=8bitgray maxscale=100 100');
call image_blob_getproperties();




------------------------------------------------
-- Exporting Images with ORDImage.export()
------------------------------------------------
connect / as sysdba
--create or replace directory imagedir as '/home/alamb/quickstart';
-- For Windows:
create or replace directory imagedir as 'c:\quickstart';
grant read on directory imagedir to scott;

--call dbms_java.grant_permission('SCOTT','java.io.FilePermission','\home\alamb\quickstart\*','WRITE');
--call dbms_java.grant_permission('ORDSYS','java.io.FilePermission','\home\alamb\quickstart\*','WRITE');

-- For Windows:
call dbms_java.grant_permission('SCOTT','java.io.FilePermission','c:\quickstart\*','WRITE');
call dbms_java.grant_permission('ORDSYS','java.io.FilePermission','c:\quickstart\*','WRITE');

connect scott/tiger
-- Writes the image data from the BLOB with id=<source_id> in image_blob_table
-- to the file named <filename> in the IMAGEDIR directory
create or replace procedure image_blob_export (source_id number, filename varchar2) as
  img_blob BLOB;
  ctx raw(64) := null;
begin
  select image_blob into img_blob from image_blob_table where id = source_id;
  ORDSYS.ORDImage.export(ctx, img_blob, 'FILE', 'IMAGEDIR', filename);
end;
/

call image_blob_export(3, 'flowers_thumbnail.jpg');
call image_blob_export(4, 'goats_grayscale.jpg');

------------------------------------------------
-- Cleaning Up
------------------------------------------------
connect / as sysdba
drop directory imagedir;
--call dbms_java.revoke_permission('SCOTT','java.io.FilePermission', '\home\alamb\quickstart\*','WRITE');
--call dbms_java.revoke_permission('ORDSYS','java.io.FilePermission', '\home\alamb\quickstart','WRITE');
-- For Windows
call dbms_java.revoke_permission('SCOTT','java.io.FilePermission','c:\quickstart\*','WRITE');
call dbms_java.revoke_permission('ORDSYS','java.io.FilePermission','c:\quickstart\*','WRITE');


connect scott/tiger
drop procedure image_blob_import;
drop procedure image_blob_getproperties;
drop procedure image_blob_processcopy;
drop procedure image_blob_export;
drop table image_blob_table;
